package com.sun.rmi.rmid;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.security.PermissionCollection;
import java.security.Permission;

public final class ExecOptionPermission extends Permission
{
    private transient boolean wildcard;
    private transient String name;
    private static final long serialVersionUID = 5842294756823092756L;
    
    public ExecOptionPermission(String name) {
        super(name);
        this.init(name);
    }
    
    public ExecOptionPermission(String s, String s2) {
        this(s);
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof ExecOptionPermission)) {
            return false;
        }
        ExecOptionPermission execOptionPermission = (ExecOptionPermission)permission;
        if (this.wildcard) {
            if (execOptionPermission.wildcard) {
                return execOptionPermission.name.startsWith(this.name);
            }
            return execOptionPermission.name.length() > this.name.length() && execOptionPermission.name.startsWith(this.name);
        }
        else {
            return !execOptionPermission.wildcard && this.name.equals(execOptionPermission.name);
        }
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o != null && o.getClass() == this.getClass() && this.getName().equals(((ExecOptionPermission)o).getName()));
    }
    
    @Override
    public int hashCode() {
        return this.getName().hashCode();
    }
    
    @Override
    public String getActions() {
        return "";
    }
    
    @Override
    public PermissionCollection newPermissionCollection() {
        return new ExecOptionPermission.ExecOptionPermissionCollection();
    }
    
    private synchronized void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.init(this.getName());
    }
    
    private void init(String name) {
        if (name == null) {
            throw new NullPointerException("name can't be null");
        }
        if (name.equals("")) {
            throw new IllegalArgumentException("name can't be empty");
        }
        if (name.endsWith(".*") || name.endsWith("=*") || name.equals("*")) {
            this.wildcard = true;
            if (name.length() == 1) {
                this.name = "";
            }
            else {
                this.name = name.substring(0, name.length() - 1);
            }
        }
        else {
            this.name = name;
        }
        return;
    }
    
    private static class ExecOptionPermissionCollection extends PermissionCollection implements Serializable
    {
        private Hashtable<String, Permission> permissions;
        private boolean all_allowed;
        private static final long serialVersionUID = -1242475729790124375L;
        
        public ExecOptionPermissionCollection() {
            this.permissions = new Hashtable(11);
            this.all_allowed = false;
        }
        
        @Override
        public void add(Permission permission) {
            if (!(permission instanceof ExecOptionPermission)) {
                throw new IllegalArgumentException("invalid permission: " + permission);
            }
            if (this.isReadOnly()) {
                throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
            }
            ExecOptionPermission execOptionPermission = (ExecOptionPermission)permission;
            this.permissions.put(execOptionPermission.getName(), permission);
            if (!this.all_allowed && execOptionPermission.getName().equals("*")) {
                this.all_allowed = true;
            }
            return;
        }
        
        @Override
        public boolean implies(Permission permission) {
            if (!(permission instanceof ExecOptionPermission)) {
                return false;
            }
            ExecOptionPermission execOptionPermission = (ExecOptionPermission)permission;
            if (this.all_allowed) {
                return true;
            }
            String s = execOptionPermission.getName();
            Permission permission2 = (Permission)this.permissions.get(s);
            if (permission2 != null) {
                return permission2.implies(permission);
            }
            int fromIndex = s.length() - 1;
            int lastIndex;
            while ((lastIndex = s.lastIndexOf(".", fromIndex)) != -1) {
                s = s.substring(0, lastIndex + 1) + "*";
                Permission permission3 = (Permission)this.permissions.get(s);
                if (permission3 != null) {
                    return permission3.implies(permission);
                }
                fromIndex = lastIndex - 1;
            }
            String key = execOptionPermission.getName();
            int fromIndex2 = key.length() - 1;
            int lastIndex2;
            while ((lastIndex2 = key.lastIndexOf("=", fromIndex2)) != -1) {
                key = key.substring(0, lastIndex2 + 1) + "*";
                Permission permission4 = (Permission)this.permissions.get(key);
                if (permission4 != null) {
                    return permission4.implies(permission);
                }
                fromIndex2 = lastIndex2 - 1;
            }
            return false;
        }
        
        @Override
        public Enumeration<Permission> elements() {
            return this.permissions.elements();
        }
    }
}
